@@ -20,12 +20,8 @@
 #include <pthread.h>
 #include <sys/uio.h>
 #include <inttypes.h>
-#include <rpc/rpc_msg.h>
 #include <glusterfs/compat.h>
-
-#ifndef MAX_IOVEC
-#define MAX_IOVEC 16
-#endif
+#include <glusterfs/client_t.h>
 
 /* TODO: we should store prognums at a centralized location to avoid conflict
          or use a robust random number generator to avoid conflicts
@@ -39,71 +35,21 @@ typedef enum {
 
 #define RPCSVC_DEFAULT_OUTSTANDING_RPC_LIMIT                                   \
     64 /* Default for protocol/server */
+#ifdef BUILD_GNFS
 #define RPCSVC_DEF_NFS_OUTSTANDING_RPC_LIMIT 16 /* Default for nfs/server */
+#endif
 #define RPCSVC_MAX_OUTSTANDING_RPC_LIMIT 65536
 #define RPCSVC_MIN_OUTSTANDING_RPC_LIMIT 0 /* No limit i.e. Unlimited */
 
 #define GF_RPCSVC "rpc-service"
-#define RPCSVC_THREAD_STACK_SIZE ((size_t)(1024 * GF_UNIT_KB))
 
-#define RPCSVC_FRAGHDR_SIZE 4 /* 4-byte RPC fragment header size */
-#define RPCSVC_DEFAULT_LISTEN_PORT GF_DEFAULT_BASE_PORT
 #define RPCSVC_DEFAULT_MEMFACTOR 8
-#define RPCSVC_EVENTPOOL_SIZE_MULT 1024
 #define RPCSVC_POOLCOUNT_MULT 64
-#define RPCSVC_CONN_READ (128 * GF_UNIT_KB)
-#define RPCSVC_PAGE_SIZE (128 * GF_UNIT_KB)
 #define RPC_ROOT_UID 0
 #define RPC_ROOT_GID 0
 #define RPC_NOBODY_UID 65534
 #define RPC_NOBODY_GID 65534
 
-/* RPC Record States */
-#define RPCSVC_READ_FRAGHDR 1
-#define RPCSVC_READ_FRAG 2
-/* The size in bytes, if crossed by a fragment will be handed over to the
- * vectored actor so that it can allocate its buffers the way it wants.
- * In our RPC layer, we assume that vectored RPC requests/records are never
- * spread over multiple RPC fragments since that prevents us from determining
- * whether the record should be handled in RPC layer completely or handed to
- * the vectored handler.
- */
-#define RPCSVC_VECTORED_FRAGSZ 4096
-#define RPCSVC_VECTOR_READCRED 1003
-#define RPCSVC_VECTOR_READVERFSZ 1004
-#define RPCSVC_VECTOR_READVERF 1005
-#define RPCSVC_VECTOR_IGNORE 1006
-#define RPCSVC_VECTOR_READVEC 1007
-#define RPCSVC_VECTOR_READPROCHDR 1008
-
-#define rpcsvc_record_vectored_baremsg(rs)                                     \
-    (((rs)->state == RPCSVC_READ_FRAG) && (rs)->vecstate == 0)
-#define rpcsvc_record_vectored_cred(rs)                                        \
-    ((rs)->vecstate == RPCSVC_VECTOR_READCRED)
-#define rpcsvc_record_vectored_verfsz(rs)                                      \
-    ((rs)->vecstate == RPCSVC_VECTOR_READVERFSZ)
-#define rpcsvc_record_vectored_verfread(rs)                                    \
-    ((rs)->vecstate == RPCSVC_VECTOR_READVERF)
-#define rpcsvc_record_vectored_ignore(rs)                                      \
-    ((rs)->vecstate == RPCSVC_VECTOR_IGNORE)
-#define rpcsvc_record_vectored_readvec(rs)                                     \
-    ((rs)->vecstate == RPCSVC_VECTOR_READVEC)
-#define rpcsvc_record_vectored_readprochdr(rs)                                 \
-    ((rs)->vecstate == RPCSVC_VECTOR_READPROCHDR)
-#define rpcsvc_record_vectored(rs) ((rs)->fragsize > RPCSVC_VECTORED_FRAGSZ)
-/* Includes bytes up to and including the credential length field. The credlen
- * will be followed by @credlen bytes of credential data which will have to be
- * read separately by the vectored reader. After the credentials comes the
- * verifier which will also have to be read separately including the 8 bytes of
- * verf flavour and verflen.
- */
-#define RPCSVC_BARERPC_MSGSZ 32
-#define rpcsvc_record_readfraghdr(rs) ((rs)->state == RPCSVC_READ_FRAGHDR)
-#define rpcsvc_record_readfrag(rs) ((rs)->state == RPCSVC_READ_FRAG)
-
-#define RPCSVC_LOWVERS 2
-#define RPCSVC_HIGHVERS 2
-
 #if 0
 #error "defined in /usr/include/rpc/auth.h"
 
@@ -138,16 +84,6 @@ typedef struct {
     struct list_head list;
 } rpcsvc_listener_t;
 
-struct rpcsvc_config {
-    int max_block_size;
-};
-
-typedef struct rpcsvc_auth_data {
-    int flavour;
-    int datalen;
-    char authdata[GF_MAX_AUTH_BYTES];
-} rpcsvc_auth_data_t;
-
 #define rpcsvc_auth_flavour(au) ((au).flavour)
 
 typedef struct drc_client drc_client_t;
@@ -171,8 +107,6 @@ struct rpcsvc_request {
 
     int procnum;
 
-    int type;
-
     /* Uid and gid filled by the rpc-auth module during the authentication
      * phase.
      */
@@ -181,7 +115,11 @@ struct rpcsvc_request {
     pid_t pid;
 
     gf_lkowner_t lk_owner;
-    uint64_t gfs_id;
+
+    /* The identifier for the call from client.
+     * Needed to pair the reply with the call.
+     */
+    uint32_t xid;
 
     /* Might want to move this to AUTH_UNIX specific state since this array
      * is not available for every authentication scheme.
@@ -195,11 +133,43 @@ struct rpcsvc_request {
      * by the program actors. This is the buffer that will need to
      * be de-xdred by the actor.
      */
-    struct iovec msg[MAX_IOVEC];
     int count;
+    struct iovec msg[MAX_IOVEC];
 
     struct iobref *iobref;
 
+    /* While the reply record is being built, this variable keeps track
+     * of how many bytes have been added to the record.
+     */
+    size_t payloadsize;
+
+    /* The credentials extracted from the rpc request */
+    client_auth_data_t cred;
+
+    /* The verified extracted from the rpc request. In request side
+     * processing this contains the verifier sent by the client, on reply
+     * side processing, it is filled with the verified that will be
+     * sent to the client.
+     */
+    client_auth_data_t verf;
+
+#if defined(BUILD_GNFS)
+    /* Container for a RPC program wanting to store a temp
+     * request-specific item. Currently this is used only
+     * by the legacy gnfs server xlator.
+     */
+    void *private;
+#endif /* BUILD_GNFS */
+
+    /* Container for transport to store request-specific item */
+    void *trans_private;
+
+    /* pointer to cached reply for use in DRC */
+    drc_cached_op_t *reply;
+
+    /* request queue in rpcsvc */
+    struct list_head request_list;
+
     /* Status of the RPC call, whether it was accepted or denied. */
     int rpc_status;
 
@@ -213,43 +183,6 @@ struct rpcsvc_request {
      */
     int auth_err;
 
-    /* There can be cases of RPC requests where the reply needs to
-     * be built from multiple sources. E.g. where even the NFS reply
-     * can contain a payload, as in the NFSv3 read reply. Here the RPC header
-     * ,NFS header and the read data are brought together separately from
-     * different buffers, so we need to stage the buffers temporarily here
-     * before all of them get added to the connection's transmission list.
-     */
-    struct list_head txlist;
-
-    /* While the reply record is being built, this variable keeps track
-     * of how many bytes have been added to the record.
-     */
-    size_t payloadsize;
-
-    /* The credentials extracted from the rpc request */
-    rpcsvc_auth_data_t cred;
-
-    /* The verified extracted from the rpc request. In request side
-     * processing this contains the verifier sent by the client, on reply
-     * side processing, it is filled with the verified that will be
-     * sent to the client.
-     */
-    rpcsvc_auth_data_t verf;
-    /* Container for a RPC program wanting to store a temp
-     * request-specific item.
-     */
-    void *private;
-
-    /* Container for transport to store request-specific item */
-    void *trans_private;
-
-    /* pointer to cached reply for use in DRC */
-    drc_cached_op_t *reply;
-
-    /* request queue in rpcsvc */
-    struct list_head request_list;
-
     /* Things passed to rpc layer from client */
 
     /* @flags: Can be used for binary data passed in xdata to be
@@ -260,10 +193,10 @@ struct rpcsvc_request {
        the one we should consider for time */
     struct timespec ctime;
 
-    /* The identifier for the call from client.
-     * Needed to pair the reply with the call.
+    /* If latency measurement is enabled, marks the request handling
+     * start time.
      */
-    uint32_t xid;
+    struct timespec begin;
 
     /* Execute this request's actor function in ownthread of program?*/
     gf_boolean_t ownthread;
@@ -362,7 +295,6 @@ typedef void (*rpcsvc_deallocate_reply)(void *msg);
  */
 typedef struct rpcsvc_actor_desc {
     char procname[RPCSVC_NAME_MAX];
-    int procnum;
     rpcsvc_actor actor;
 
     /* Handler for cases where the RPC requests fragments are large enough
@@ -375,18 +307,20 @@ typedef struct rpcsvc_actor_desc {
      */
     rpcsvc_vector_sizer vector_sizer;
 
+    int procnum;
+
     /* Can actor be ran on behalf an unprivileged requestor? */
-    gf_boolean_t unprivileged;
     drc_op_type_t op_type;
+    gf_boolean_t unprivileged;
 } rpcsvc_actor_t;
 
 typedef struct rpcsvc_request_queue {
-    int gen;
     struct list_head request_queue;
     pthread_mutex_t queue_lock;
     pthread_cond_t queue_cond;
     pthread_t thread;
     struct rpcsvc_program *program;
+    int gen;
     gf_boolean_t waiting;
 } rpcsvc_request_queue_t;
 
@@ -421,10 +355,9 @@ struct rpcsvc_program {
     int numactors;          /* Num actors in actor array */
     int proghighvers;       /* Highest ver for program
                                supported by the system. */
-    int proglowvers;        /* Lowest ver */
-
     /* Program specific state handed to actors */
     void *private;
+    gf_latency_t *latencies; /*Tracks latency statistics for the rpc call*/
 
     /* This upcall is provided by the program during registration.
      * It is used to notify the program about events like connection being
@@ -434,6 +367,8 @@ struct rpcsvc_program {
      */
     rpcsvc_notify_t notify;
 
+    int proglowvers; /* Lowest ver */
+
     /* An integer that identifies the min auth strength that is required
      * by this protocol, for eg. MOUNT3 needs AUTH_UNIX at least.
      * See RFC 1813, Section 5.2.1.
@@ -443,9 +378,7 @@ struct rpcsvc_program {
     /* list member to link to list of registered services with rpcsvc */
     struct list_head program;
     rpcsvc_request_queue_t request_queue[EVENT_MAX_THREADS];
-    char request_queue_status[EVENT_MAX_THREADS / 8 + 1];
     pthread_mutex_t thr_lock;
-    pthread_cond_t thr_cond;
     int threadcount;
     int thr_queue;
     pthread_key_t req_queue_key;
@@ -462,6 +395,7 @@ struct rpcsvc_program {
     gf_boolean_t alive;
 
     gf_boolean_t synctask;
+    unsigned long request_queue_status[EVENT_MAX_THREADS / __BITS_PER_LONG];
 };
 
 typedef struct rpcsvc_cbk_program {
@@ -569,14 +503,6 @@ rpcsvc_transport_privport_check(rpcsvc_t *svc, char *volname, uint16_t port);
 #define rpcsvc_request_seterr(req, err) ((req)->rpc_err = (int)(err))
 #define rpcsvc_request_set_autherr(req, err) ((req)->auth_err = (int)(err))
 
-extern int
-rpcsvc_submit_vectors(rpcsvc_request_t *req);
-
-extern int
-rpcsvc_request_attach_vector(rpcsvc_request_t *req, struct iovec msgvec,
-                             struct iobuf *iob, struct iobref *ioref,
-                             int finalvector);
-
 typedef int (*auth_init_trans)(rpc_transport_t *trans, void *priv);
 typedef int (*auth_init_request)(rpcsvc_request_t *req, void *priv);
 typedef int (*auth_request_authenticate)(rpcsvc_request_t *req, void *priv);
@@ -593,9 +519,9 @@ typedef struct rpcsvc_auth_ops {
 
 typedef struct rpcsvc_auth_flavour_desc {
     char authname[RPCSVC_NAME_MAX];
-    int authnum;
     rpcsvc_auth_ops_t *authops;
     void *authprivate;
+    int authnum;
 } rpcsvc_auth_t;
 
 typedef void *(*rpcsvc_auth_initer_t)(rpcsvc_t *svc, dict_t *options);
@@ -624,9 +550,6 @@ rpcsvc_auth_init(rpcsvc_t *svc, dict_t *options);
 extern int
 rpcsvc_auth_reconf(rpcsvc_t *svc, dict_t *options);
 
-extern int
-rpcsvc_auth_transport_init(rpc_transport_t *xprt);
-
 extern int
 rpcsvc_authenticate(rpcsvc_request_t *req);
 
@@ -690,4 +613,6 @@ rpcsvc_autoscale_threads(glusterfs_ctx_t *ctx, rpcsvc_t *rpc, int incr);
 
 extern int
 rpcsvc_destroy(rpcsvc_t *svc);
+void
+rpcsvc_statedump(rpcsvc_t *svc);
 #endif
